package edu.lawrence.androidweather

import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory

data class TempRange(
    val day : Double,
    val min : Double?,
    val max : Double?,
    val night : Double,
    val eve : Double,
    val morn : Double
)

data class Weather(
    val id: Int,
    val main: String,
    val description: String,
    val icon: String)

data class ForecastDay(
    val dt: Int,
    val sunrise: Int,
    val sunset: Int,
    val moonrise: Int,
    val moonset: Int,
    val moon_phase: Double,
    val temp: TempRange,
    val feels_like: TempRange,
    val pressure: Int,
    val humidity: Int,
    val dew_point: Double,
    val wind_speed: Double,
    val wind_deg: Int,
    val wind_gust: Double,
    val weather: List<Weather>,
    val clouds: Int,
    val pop: Double,
    val rain: Double?,
    val uvi: Double)

data class Forecast(
    val lat : Double,
    val lon : Double,
    val timezone : String,
    val timezone_offset : Int,
    val daily : List<ForecastDay>)

class WeatherModel : ViewModel() {
    private lateinit var restInterface : WeatherApiService
    val location = mutableStateOf(LocationPreferences.getLocation())
    val forecast = mutableStateOf<Forecast?>(null)

    init {
        val retrofit : Retrofit = Retrofit.Builder()
            .addConverterFactory(GsonConverterFactory.create())
            .baseUrl("https://api.openweathermap.org/")
            .build()
        restInterface = retrofit.create(WeatherApiService::class.java)
    }

    fun getForecast() {
        val latitude = location.value.latitude
        val longitude = location.value.longitude
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val result = restInterface.getForecast(
                    latitude = latitude,
                    longitude = longitude,
                    key = KeyHolder.apikey
                )
                withContext(Dispatchers.Main) {
                    forecast.value = result
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }

    fun setLocationZip(zip : String) {
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val result = restInterface.getLocation(zip=zip+",US",key = KeyHolder.apikey)
                val newLocation = Location(zip = zip,
                    latitude = result.lat.toFloat(),
                    longitude = result.lon.toFloat(),
                    name = result.name)
                LocationPreferences.setLocation(newLocation)
                withContext(Dispatchers.Main) {
                    location.value = newLocation
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }
}